<?php

/**
 * @file
 * Contains \Drupal\inspect\Form\InspectSettingsForm.
 */

namespace Drupal\inspect\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\inspect\Inspect;

/**
 * Configure inspect settings for this site.
 */
class InspectSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'inspect_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'inspect.settings'
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('inspect.settings');

    // Establish max/default values for outmax lengths.
    $output_max_measured = Inspect::outputMax(TRUE);
    $output_max_default = min($output_max_measured, Inspect::OUTPUT_DEFAULT);
    $output_max_absolute = min($output_max_measured, Inspect::OUTPUT_MAX);

    $form['help'] = array(
      '#type' => 'markup',
      '#markup' => '<p>Inspect\'s ' . t('<a href="!href-inspect.help">help page</a> and <a href="!href-inspect.permissions">permissions</a>.', array('!href-inspect.help' => '/admin/help/inspect', '!href-inspect.permissions' => \Drupal::url('user.admin_permissions', array(), array('fragment' => 'module-inspect')))) . '</p>'
    );

    $form['output_fuses'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Fuses - graceful degradation'),
      '#description' => $this->t(
        'Maximum output lengths of dumps and traces secure that Inspect degrades gracefully when analyzing an overly long data set.
!breakThe variable inspector will make a second attempt when reaching maximum output (using less depth and higher truncation),
!breakwhereas the tracer automatically reduces stack frame limit when reaching maximum output.
!breakInspections and traces also abort (cleanly) if more than a certain percent of PHP\'s max execution time has passed.'
        , array('!break' => '<br/>')
      ),
      'inspect_output_max' => array(
        '#type' => 'textfield',
        '#title' => $this->t('Maximum output length'),
        '#description' => $this->t(
          'Absolute max is !max. Measured max is MySQL max_allowed_packet: !measured bytes ~ !mb Mb.!breakActual max will be slightly shorter to allow for metadata and database query.!breakDefault: !default.',
          array('!max' => $output_max_absolute, '!measured' => $output_max_measured, '!mb' => round($output_max_measured / 1048576, 1), '!default' => $output_max_default, '!break' => '<br/>')
        ),
        // If no previous setting, and the suggested setting is larger than
        // default: use default.
        '#default_value' => ($v = $config->get('output_max')) !== NULL ? $v : $output_max_default,
        '#size' => 8,
        '#required' => TRUE,
      ),
      'inspect_exectime_percent' => array(
        '#type' => 'textfield',
        '#title' => $this->t(
          'Abort variable/trace inspection if called later than this percent of max execution time (@max_execution_time seconds)',
          array('@max_execution_time' => ini_get('max_execution_time'))
        ),
        '#description' => $this->t(
          'If PHP max_execution_time is being changed during run-time - set_time_limit() - this \'fuse\' will not work predictably.
!breakBecause max_execution_time is only being checked at first inspection/trace in a request,
!breakand any later changes will not be reflected by Inspect\'s time limit \'fuse\' in that request.',
          array('!break' => '<br/>')
        ),
        '#default_value' => ($v = $config->get('exectime_percent')) !== NULL ? $v : 90,
        '#size' => 2,
        '#required' => TRUE,
      ),
    );

    $severities = RfcLogLevel::getLevels();
    $form['inspection_tracing'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Inspection and tracing'),
      'inspect_truncate' => array(
        '#type' => 'textfield',
        '#title' => $this->t('Default string truncation'),
        '#description' => $this->t(
          'Maximum: !max. Default: !default.',
          array('!max' => Inspect::TRUNCATE_MIN, '!default' => Inspect::TRUNCATE_DEFAULT)
        ),
        '#default_value' => ($v = $config->get('truncate')) !== NULL ? $v : Inspect::TRUNCATE_DEFAULT,
        '#size' => 5,
        '#required' => TRUE,
      ),
      'inspect_trace_limit' => array(
        '#type' => 'textfield',
        '#title' => $this->t('Error/back tracer\'s default stack frame limit'),
        '#description' => $this->t(
          'Recommended value: !default; a higher a value (deeper stack tracing) will rarely provide additional usable information.',
          array('!default' => Inspect::TRACE_LIMIT_DEFAULT)
        ),
        '#default_value' => ($v = $config->get('trace_limit')) !== NULL ? $v : Inspect::TRACE_LIMIT_DEFAULT,
        '#size' => 2,
        '#required' => TRUE,
      ),
      'inspect_fronttoback_sevmax' => array(
        '#type' => 'select',
        '#title' => $this->t('Maximum severity when frontend logs to backend'),
        '#description' => $this->t(
          'A frontend event having higher severity will be logged using this severity. Default: !default.',
          array('!default' => $severities[RfcLogLevel::ERROR])
        ),
        '#options' => $severities,
        '#default_value' => ($v = $config->get('fronttoback_sevmax')) !== NULL ? $v : RfcLogLevel::ERROR,
        '#attributes' => array(
          'autocomplete' => 'off',
        ),
      ),
      'inspect_session_counters' => array(
        '#type' => 'checkbox',
        '#title' => $this->t('Enable session counters'),
        '#description' => $this->t('Not recommendable in daily use on a production site, because this feature uses and updates a cookie, and it also adds a slight performance hit to each and every request.'),
        '#default_value' => ($v = $config->get('session_counters')) !== NULL ? $v : 0,
        '#attributes' => array(
          'autocomplete' => 'off',
        ),
      ),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $output_max_absolute = min(Inspect::outputMax(TRUE), Inspect::OUTPUT_MAX);

    $output_max = $form_state->getValue('inspect_output_max');
    if (!$output_max) {
      $form_state->setErrorByName(
        'inspect_output_max',
        $this->t('Maximum output length cannot be empty.')
      );
    }
    elseif (!ctype_digit($output_max)) {
      $form_state->setErrorByName(
        'inspect_output_max',
        $this->t('Maximum output length must be a (positive) integer.')
      );
    }
    elseif ($output_max > $output_max_absolute) {
      $form_state->setErrorByName(
        'inspect_output_max',
        $this->t('Maximum output length cannot be more than !max.', array('!max' => $output_max_absolute))
      );
    }

    $exectime_percent = $form_state->getValue('inspect_exectime_percent');
    if (!$exectime_percent) {
      $form_state->setErrorByName(
        'inspect_exectime_percent',
        $this->t('Percent of max execution time cannot be empty.')
      );
    }
    elseif (!ctype_digit($exectime_percent)) {
      $form_state->setErrorByName(
        'inspect_exectime_percent',
        $this->t('Percent of max execution must be a (positive) integer.')
      );
    }
    elseif ($exectime_percent < 10) {
      $form_state->setErrorByName(
        'inspect_exectime_percent',
        $this->t('Percent of max execution cannot be less than 10.')
      );
    }
    elseif ($exectime_percent > 95) {
      $form_state->setErrorByName(
        'inspect_exectime_percent',
        $this->t('Percent of max execution cannot be more than 95.')
      );
    }

    $truncate = $form_state->getValue('inspect_truncate');
    if (!ctype_digit($truncate)) {
      $form_state->setErrorByName(
        'inspect_truncate',
        $this->t('Default string truncation must be a (positive) integer.')
      );
    }
    elseif ($truncate > Inspect::TRUNCATE_MIN) {
      $form_state->setErrorByName(
        'inspect_truncate',
        $this->t('Default string truncation cannot be more than !max.', array('!max' => Inspect::TRUNCATE_MIN))
      );
    }

    $trace_limit = $form_state->getValue('inspect_trace_limit');
    if (!$trace_limit) {
      $form_state->setErrorByName(
        'inspect_trace_limit',
        $this->t('Default stack frame limit cannot be empty.')
      );
    }
    elseif (!ctype_digit($trace_limit)) {
      $form_state->setErrorByName(
        'inspect_trace_limit',
        $this->t('Default stack frame limit must be a (positive) integer.')
      );
    }
    elseif ($trace_limit > 100) {
      $form_state->setErrorByName(
        'inspect_trace_limit',
        $this->t('Default stack frame limit cannot be more than 100.')
      );
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $names = array(
      'output_max',
      'exectime_percent',
      'truncate',
      'trace_limit',
      'fronttoback_sevmax',
      'session_counters',
    );

    $config = $this->config('inspect.settings');

    foreach ($names as $name) {
      // All vars are integers.
      $config->set($name, (int) $form_state->getValue('inspect_' . $name));
    }
    $config->save();

    parent::submitForm($form, $form_state);
  }

}

