<?php
/**
 * @file
 * Contains \Drupal\inspect\Inspect.
 */

namespace Drupal\inspect;

use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\StreamWrapper\PrivateStream;
use Drupal\Component\Utility\SafeMarkup;
use Drupal\Component\Utility\Unicode;

include_once __DIR__ . '/../vendor/simplecomplex/inspect/src/Inspect.php';

/**
 * Drupal 8 specialization of SimpleComplex Inspect.
 */
class Inspect extends \SimpleComplex\Inspect\Inspect {

  /**
   * Checks if user is allowed to output to that target.
   *
   *  Special rules:
   *  - filing uses same permissions as logging
   *  - drush/CLI is always allowed to get|log|file
   *  - target 'frontend log'
   *
   * @param string $target
   *   Values: log|file|frontend log|get.
   *   Default: get.
   *
   * @return boolean
   */
  public static function permit($target = 'get') {
    static $permit = array(
      'get' => NULL,
      'log' => NULL,
      'file' => NULL,
      'frontend log' => NULL,
    );

    if (($perm = $permit[$target]) || $perm === FALSE) {
      return $perm;
    }

    if (PHP_SAPI === 'cli') {
      $permit = array(
        'get' => TRUE,
        'log' => TRUE,
        'file' => TRUE,
        'frontend log' => FALSE, // Doesn't make sense.
      );
      return $permit[$target];
    }

    $perm = FALSE;
    $user = \Drupal::currentUser();
    switch ('' . $target) {
      case 'log':
      case 'file':
        if ($user->hasPermission('inspect log')) {
          $perm = TRUE;
        }
        $permit['log'] = $perm;
        $permit['file'] = $perm;
        break;
      case 'frontend log':
        // A subset of 'log'.
        if ($user->hasPermission('inspect log') && $user->hasPermission('inspect frontend log')) {
          $perm = TRUE;
        }
        $permit['frontend log'] = $perm;
        break;
      case 'get':
        if ($user->hasPermission('inspect get')) {
          $perm = TRUE;
        }
        $permit['get'] = $perm;
        break;
      default:
    }
    return $perm;
  }

  /**
   * Get request time start - in milliseconds.
   *
   * @return float
   */
  public static function requestTimeMilli() {
    if (!($rtm =& static::$requestTimeMilli)) {
      $rtm = round($_SERVER['REQUEST_TIME_FLOAT'] * 1000, 3);
    }
    return $rtm;
  }

  //  Utility methods.

  /**
   * Get maximum output length.
   *
   * Sets max. in 'inspect.output_max' if that variable is empty.
   *
   * @see Inspect::OUTPUT_DEFAULT
   * @see Inspect::OUTPUT_MAX
   *
   * @param boolean $check
   *   Truthy: attempts to read database max query length; fall-back
   *   Inspect::OUTPUT_DEFAULT.
   *   Default: FALSE (~ gets from variable setting (or database max query
   *   length) and subtracts some for safe margin).
   *
   * @return integer
   */
  public static function outputMax($check = FALSE) {
    $setting = static::configGet('inspect', 'output_max', 0);

    if ($setting && !$check) {
      // Subtract 5% for database abstraction query manipulation
      // (seen 1.5% difference).
      // Subtract 500 as metadata margin.
      return (int)floor($setting * 0.95) - 500;
    }

    try {
      if (!($le = (int) db_query('SELECT @@max_allowed_packet')->fetchField())) {
        $le = static::OUTPUT_DEFAULT;
      }
      elseif ($le > static::OUTPUT_MAX) {
        $le = static::OUTPUT_MAX;
      }
    }
    catch (\Exception $xc) { // Possible PDOException.
      static::logToStandard('Failed to read max query length of (MySQL type) database', 'inspect', RfcLogLevel::WARNING);
      $le = static::OUTPUT_DEFAULT;
    }

    if (!$setting) {
      // If no previous setting, and the suggested setting is larger than
      // default: use default.
      static::configSet('inspect', 'output_max', $le > static::OUTPUT_DEFAULT ? static::OUTPUT_DEFAULT : $le);
    }

    if (!$check) {
      // Subtract 5% for database abstraction query manipulation
      // (seen 1.5% difference).
      // Subtract 500 as metadata margin.
      return (int)floor($le * 0.95) - 500;
    }

    return $le;
  }

  /**
   * Secures existance of filing directory; optionally subdir to that.
   *
   * Uses private://module/inspect.
   *
   * @param string $sub_dir
   *   Default: empty.
   *   No leading nor trailing slash.
   * @param integer $mode
   *   Default: 0775 (~ group write mode).
   *
   * @return string|boolean
   *   FALSE: on error.
   */
  public static function ensureDirectory($sub_dir = '', $mode = 0775) {
    static $_dir;
    if (!($dir = $_dir)) {
      if ($dir === NULL) {
        // ~ $settings: file_private_path; apparantly no way of getting it
        // from config.
        // \Drupal\Core\StreamWrapper\PrivateStream.
        if (!($dir = PrivateStream::basePath())) {
          static::logToStandard(
            'Private files directory not defined, check /admin/config/media/file-system', 'inspect', RfcLogLevel::WARNING
          );
          return ($_dir = FALSE);
        }
      }
      else {
        return FALSE;
      }
      $le = strlen($dir);
      if ($dir{$le - 1} == '/') { // Remove trailing slash.
        $dir = substr($dir, 0, $le - 1); // Deliberately not multibyte substr().
        $le -= 1;
      }
      if ($le < 2) { // Private files dir cannot be shorter than 2 chars (/x).
        return ($_dir = FALSE);
      }
      if ($le < 2
        || (!is_dir($dir .= '/module/inspect') && !mkdir($dir, $mode, TRUE))
      ) {
        static::logToStandard(
          $le < 2 ? 'Private files directory cannot be shorter than 2 chars, check /admin/config/media/file-system' :
            'Directory private://module/inspect is not a dir or cannot be created',
          'inspect',
          RfcLogLevel::WARNING
        );
        return ($_dir = FALSE);
      }
      $_dir = $dir;
    }
    if (!$sub_dir
      || is_dir($dir .= '/' . $sub_dir) || mkdir($dir, $mode, TRUE)
    ) {
      return $dir;
    }
    static::logToStandard(
      'Directory private://module/inspect/' . $sub_dir . ' is not a dir or cannot be created',
      'inspect',
      RfcLogLevel::WARNING
    );
    return FALSE; // Failed to ensure subdir.
  }

  /**
   * Log to Drupal's logger.
   *
   * Caller must guarantee to escape type and message.
   *
   * May truncate message to prevent failure.
   *
   * Uses the watchdog 'link' bucket for recording $code.
   *
   * @see \Drupal\dblog\Logger\DbLog::log()
   *
   * @param string $message
   *   Default: empty string.
   * @param string $type
   *   Default: inspect.
   * @param integer|string $severity
   *   Default: 'debug'.
   * @param integer|string $code
   *   Default: zero.
   *
   * @return boolean
   */
  protected static function logToStandard($message = '', $type = 'inspect', $severity = 'debug', $code = 0) {
    // Truncate.
    // No instance output_max here, so we have to resort to the, possibly
    // larger, class value (log() and trace() truncates by instance output_max).
    // Deliberately not multibyte strlen().
    if (strlen($message) > static::$outputMax) {
      $message = static::truncateBytes($message, static::$outputMax - 5) . '[...]';
    }

    $context = array(
      'type' => $type,
    );
    if ($code) {
      $context['code'] = $context['link'] = $code;
    }
    try {
      \Drupal::logger($type)->log(static::severity($severity), $message, $context);

      return TRUE;
    }
    catch (\Exception $xc) {
    }

    return FALSE;
  }

  /**
   * @param string $str
   *
   * @return boolean
   */
  protected static function validUtf8($str) {
    return Unicode::validateUtf8($str);
  }

  /**
   * @param string $str
   *
   * @return string
   */
  protected static function plaintext($str) {
    return SafeMarkup::checkPlain(strip_tags($str));
  }

  /**
   * Multibyte-safe string length.
   *
   * @param string $str
   *
   * @return integer
   */
  protected static function mb_strlen($str) {
    return Unicode::strlen($str);
  }

  /**
   * Multibyte-safe sub string.
   *
   * @param string $str
   * @param integer $start
   * @param integer|NULL $length
   *   Default: NULL.
   *
   * @return string
   */
  protected static function mb_substr($str, $start, $length = NULL) {
    return Unicode::substr($str, $start, $length);
  }

  /**
   * Truncate multibyte safe until ASCII length is equal to/less than arg
   * length.
   *
   * @param string $str
   * @param integer $length
   *   Fails if non-integer (like float or string) and PHP>=5.4.
   *
   * @return string
   */
  protected static function truncateBytes($str, $length) {
    return Unicode::truncateBytes($str, $length);
  }

  /**
   * @return string
   */
  protected static function docRoot() {
    return DRUPAL_ROOT;
  }

  /**
   * @param string $domain
   * @param string $name
   * @param mixed $default
   *   Default: NULL.
   *
   * @return mixed
   */
  public static function configGet($domain, $name, $default = NULL) {
    $value =
      /** @var \Drupal\Core\Config\ImmutableConfig */
      \Drupal::config($domain == 'inspect' ? 'inspect.settings' : $domain)
      ->get($name);
    return $value === NULL ? $default : $value;
  }

  /**
   * Sets for current request and saves permanently.
   *
   * @param string $domain
   * @param string $name
   * @param mixed $value
   */
  protected static function configSet($domain, $name, $value) {
    /** @var \Drupal\Core\Config\Config */
    \Drupal::configFactory()->getEditable($domain == 'inspect' ? 'inspect.settings' : $domain)
      // For now.
      ->set($name, $value)
      // For later.
      ->save();
  }

  /**
   * @param string $name
   * @param mixed $value
   * @param integer $expire
   *   Default: zero (~ session).
   * @param boolean $httponly
   *   Default: FALSE.
   */
  protected static function cookieSet($name, $value, $expire = 0, $httponly = FALSE) {
    // @todo: Get and set cookie via Symfony\Component\HttpFoundation\Cookie or GuzzleHttp\Cookie or something.
    @setcookie(
      $name, '' . $value, $expire,
      base_path(), '', !empty($_SERVER['HTTPS']), $httponly
    );
  }

  /**
   * @return integer|string
   */
  protected static function userId() {
    return \Drupal::currentUser()->id();
  }

}
